/**
 * 
 * we call each option with valid and invalid option.
 * each command group has its valid set of commands.
 * 
 * the options are broken into following things
 * 
 * diagnostic commands.:
 * 	i, t, s, l, L, p, x, d
 * 	valid option is : eth0, sda, scsi_device, etc.
 * 	invaid option is : op_inval
 * 	
 * 	
 *
 * list commands:
 * 	b, c, r, v
 * 	
 *	valid options:
 *		b -> pci, scsi, usb, etc.
 *		c -> net, scsi_host, scsi_device, etc.
 *		r -> pci0000:00, system etc.
 *		v 
 *	inval options: opt_inval.
 *	
 * 	
 *
 * misc command:
 * 	R, h
 * 	this has no inval option, the user has to be root to invoke
 * 	the -R option.
 *
 */

#include <stdio.h>
#include <unistd.h>


/*
 * GLOBALS are here..
 */
#define VAL_CLASS_NAME		"net"
#define VAL_CLASS_DEVICE_NAME	"eth0"
#define VAL_BUS_NAME		"scsi"
#define VAL_BUS_DEVICE_NAME	"0:0:0:0"
#define VAL_BLOCK_NAME		"block"
#define VAL_BLOCK_DEVICE_NAME	"sda"
#define VAL_ROOT_NAME		"pci0000:00"
#define VAL_ROOT_DEVICE_NAME	"0000:00:00.0"
#define INVAL_NAME		"INVALID_NAME"
#define INVAL_BUS		"INVALID_BUS"
#define INVAL_CLASS		"INVALID_CLASS"
#define INVAL_ROOT		"INVALID_ROOT"

#define SYSDIAG			"sysdiag"
#define ECH0			" >> "

FILE *my_stdout;
char *file_name = NULL;

int run_me(char *cmd)
{
	strcat(cmd, ECH0);
	strcat(cmd, file_name);
	system(cmd);
}
int log_me(char *cmd)
{
	char log[255];
	
	memset(log, 0, 255);
	sprintf(log, "echo \"***************************************************\" >> %s", file_name);
	system(log);
	
	memset(log, 0, 255);
	sprintf(log, "echo \"Testing:%s\" >> %s", cmd, file_name);
	system(log);
	
	memset(log, 0, 255);
	sprintf(log, "echo \"***************************************************\" >> %s", file_name);
	system(log);
}

/**
 * we test out, -d, -h, -R options
 *
 *
 */
int test_rest()
{
	char cmd[255];
	char *ptr = NULL;
	int len = 0;

	memset(cmd, 0, 255);
	strcpy(cmd, SYSDIAG);
	strcat(cmd, " -");
	strcat(cmd, "h");
	len = strlen(cmd);
	ptr = cmd;
	ptr += len;

	log_me(cmd);
	run_me(cmd);
	memset(ptr, 0, 255 -len);

	cmd[len-1] = 'd';
	log_me(cmd);
	
	run_me(cmd);
	memset(ptr, 0, 255 -len);
	cmd[len-1] = 'R';
	log_me(cmd);
	
	run_me(cmd);
	return 0;

}

/**
 *  -r  root-name
 *  -r  root_name root-device-name
 *
 *
 */

int test_root_opts(char *cmd, int flag)
{
	char *ptr = NULL;
	char log[255];
	int len;
	
	if (cmd == NULL) {
		return -1;
	}
	memset(log, 0, 255);
	ptr = cmd;
	/*
	 * test the -x option..
	 * 
	 */
	strcat(cmd, "r");
	len = strlen(cmd);
	ptr += len;
	
	if (flag == 0)
		strcat(cmd, VAL_ROOT_NAME);
	else 
		strcat(cmd, INVAL_BUS);

	log_me(cmd);
	run_me(cmd);
	fflush(my_stdout);
	fflush(stderr);
	
	memset(ptr, 0, 255 -len);
	strcat(cmd, " ");
	
	if (flag == 0)
		strcat(cmd, VAL_ROOT_DEVICE_NAME);
	else 
		strcat(cmd, INVAL_NAME);
	
	log_me(cmd);
	run_me(cmd);
	memset(ptr, 0, 255 -len);
	
	fflush(my_stdout);
	fflush(stderr);

	return 0;
}


/**
 *  -c  class-name
 *  -c  class_name class-device-name
 *  -cx class-name
 *
 *
 */

int test_class_opts(char *cmd, int flag)
{
	char *ptr = NULL;
	char log[255];
	int len;
	
	if (cmd == NULL) {
		return -1;
	}
	memset(log, 0, 255);
	ptr = cmd;
	/*
	 * test the -x option..
	 * 
	 */
	strcat(cmd, "c");
	len = strlen(cmd);
	ptr += len;
	strcat(cmd, "x ");
	
	if (flag == 0)
		strcat(cmd, VAL_CLASS_NAME);
	else 
		strcat(cmd, INVAL_CLASS);
	
	log_me(cmd);
//	sprintf(log, "echo \"Testing CLASS opts %s\" >> %s", cmd, file_name);
//	system(log);

	run_me(cmd);
	
	fflush(my_stdout);
	fflush(stderr);
	
	memset(ptr, 0, 255 -len);
	strcat(cmd, " ");
	
	if (flag == 0)
		strcat(cmd, VAL_CLASS_NAME);
	else 
		strcat(cmd, INVAL_CLASS);

	log_me(cmd);
//	sprintf(log, "echo \"Testing CLASS opts %s\" >> %s", cmd, file_name);
//	system(log);

	len = strlen(cmd);
	ptr = cmd;
	ptr += len;
	
	run_me(cmd);
	
	fflush(my_stdout);
	fflush(stderr);
	memset(ptr, 0, 255 - len);

	strcat(cmd, " ");
	if (flag == 0)
		strcat(cmd, VAL_CLASS_DEVICE_NAME);
	else 
		strcat(cmd, INVAL_NAME);
	
	log_me(cmd);
//	sprintf(log, "echo \"Testing CLASS opts %s\" >> %s", cmd, file_name);
//	system(log);

	run_me(cmd);
	memset(ptr, 0, 255 -len);
	
	fflush(my_stdout);
	fflush(stderr);

	return 0;
}



/**
 *  -b  bus-name
 *  -b  bus_name bus-device-name
 *  -bx bus-name
 *
 *
 */

int test_bus_opts(char *cmd, int flag)
{
	char *ptr = NULL;
	char log[255];
	int len;
	
	if (cmd == NULL) {
		return -1;
	}
	memset(log, 0, 255);
	ptr = cmd;
	/*
	 * test the -x option..
	 * 
	 */
	strcat(cmd, "b");
	len = strlen(cmd);
	ptr += len;
	strcat(cmd, "x ");
	
	if (flag == 0)
		strcat(cmd, VAL_BUS_NAME);
	else 
		strcat(cmd, INVAL_BUS);
	
	log_me(cmd);
//	sprintf(log, "echo \"Testing opts %s\" >> %s", cmd, file_name);
//	system(log);

	run_me(cmd);
	
	fflush(my_stdout);
	fflush(stderr);
	
	memset(ptr, 0, 255 -len);
	strcat(cmd, " ");
	
	if (flag == 0)
		strcat(cmd, VAL_BUS_NAME);
	else 
		strcat(cmd, INVAL_BUS);

	log_me(cmd);
//	sprintf(log, "echo \"Testing opts %s\" >> %s", cmd, file_name);
//	system(log);

	len = strlen(cmd);
	ptr = cmd;
	ptr += len;
	
	run_me(cmd);
	
	fflush(my_stdout);
	fflush(stderr);
	memset(ptr, 0, 255 - len);

	strcat(cmd, " ");
	if (flag == 0)
		strcat(cmd, VAL_BUS_DEVICE_NAME);
	else 
		strcat(cmd, INVAL_NAME);
	
	log_me(cmd);
//	sprintf(log, "echo \"Testing opts %s\" >> %s", cmd, file_name);
//	system(log);

	run_me(cmd);
	memset(ptr, 0, 255 -len);
	
	fflush(my_stdout);
	fflush(stderr);

	return 0;
}


/**
 *
 * the various options with diag options are
 * 
 *  -x
 *  -b   bus-name
 *  -bx  bus-name
 *  -b   bus-name class-device-name
 *  -c   class name
 *  -cx  class name
 *  -c   class name class-device-name
 *  -p 
 *  
 *  
 * 
 */
int test_diag_opt(char *opt)
{
	char cmd[255];
	char log[255];
	char *ptr = NULL;
	int i = 0, len =0;
	

	memset(cmd, 0, 255);
	memset(log, 0, 255);
	ptr = cmd;
	strcpy(cmd, SYSDIAG);
	strcat(cmd, " -");
	strcat(cmd, opt);
	len = strlen(cmd);
	ptr+=len;
	/*
	 * test the -x option..
	 * 
	 */
	strcat(cmd, "x");
	log_me(cmd);
//	sprintf(log, "echo \"Testing opts %sx\" >> %s", opt, file_name);
//	system(log);
	run_me(cmd);
	fflush(my_stdout);
	fflush(stderr);

	/*
	 * test with the bus options here.
	 */
	for (i =0; i < 2; i++) {
		memset(ptr, 0, 255 - len);
		test_bus_opts(cmd, i);
//		memset(ptr, 0, 255 - len);
//		test_class_opts(cmd, i);
	}
		
	return 0;
}
/**
 * we test the list by passing the 
 *
 */
int test_list()
{
	int i = 0, len = 0;
	char cmd[255];
	char *ptr = NULL;

	memset(cmd, 0, 255);
	strcpy(cmd, SYSDIAG);
	strcat(cmd, " -v");
	len = strlen(cmd);
	ptr = cmd;
	ptr+=len;

	log_me(cmd);
	run_me(cmd);
	for (i =0; i < 2; i++) {
		memset(ptr, 0, 255 - len);
		test_bus_opts(cmd, i);
		memset(ptr, 0, 255 - len);
		test_class_opts(cmd, i);
		memset(ptr, 0, 255 - len);
		test_root_opts(cmd, i);
	}
		



}

int test_diag_opts()
{
	char *opts[] = { "i" , "t", "s", "l", "L", NULL};
	int i = 0;

	for (i = 0; opts[i] != NULL ; i++) {
		test_diag_opt(opts[i]);		
	}

}


int main(int argc, char *argv[])
{
	char std_out[] = "/dev/stdout";
	
	if (argc > 1) 
		file_name = argv[1];
	else 
		file_name = std_out;

	my_stdout = fopen(file_name,"w");

	test_diag_opts();
	test_list();
	test_rest();
	return 0;


}
